{ Photometry Express
  Main Form
  by J C Moore
  Fleet Electronics Consultancy
  Last update 05 Mar 2013
  }
//----------------------------------------------------------------------------
//
// Module: unit_main.pas (Part of Photometry Express)
// Author: J C Moore, Fleet Electronics Consultancy
//
// Copyright (C) 2013  J C Moore
//
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation, either version 3 of
// the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// see http://www.gnu.org/licenses/
//
// Update History:
//    05 Mar 2013: last update by J C Moore
//
//
//----------------------------------------------------------------------------
unit Unit_Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, StdCtrls, Grids, ExtCtrls, Spin;

const
  MAX_OBJECTS = 200;
  INIFILENAME = '\photometry_express.ini';
  {$I 'Photo_Express_Errors.inc'}

type
  TSkyPosition = record
    RA			: Double;     // hours
    Dec			: Double;     // degrees
  end;

  TCCDPosition = Record
    x   : Double;
    y   : Double;
  end;

  TTargetData = record
    Name      : String;
    RA        : Double;
    Dec       : Double;
    RName     : String;
    RRA       : Double;
    RDec      : Double;
    RFilter   : Char;
    Rmag      : Double;
    Threshold : Double;
    ChartName : String;
  end;

  TMain_Form = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Open1: TMenuItem;
    OpenDialog1: TOpenDialog;
    ComboBox_Scale: TComboBox;
    Button_Plus: TButton;
    Button_Minus: TButton;
    Button_Visualisation: TButton;
    StringGrid_FITs: TStringGrid;
    Panel_Bottom: TPanel;
    Panel_Location: TPanel;
    Panel_Pixel: TPanel;
    Panel_BP: TPanel;
    Panel_WP: TPanel;
    Panel_Position: TPanel;
    Panel_Targets: TPanel;
    Panel_Chart: TPanel;
    Panel_Magnitude: TPanel;
    Panel_Outburst: TPanel;
    Panel_Reference: TPanel;
    Panel_STDEV: TPanel;
    Panel_Plate_Solved: TPanel;
    Panel_Image_Filter: TPanel;
    Panel_Errors: TPanel;
    FITs1: TMenuItem;
    ShowHeader1: TMenuItem;
    Close1: TMenuItem;      // FITs header
    Close2: TMenuItem;      // Image
    LoadData1: TMenuItem;
    CloseAll1: TMenuItem;
    Settings1: TMenuItem;
    Button_AAVSO: TButton;
    Timer1: TTimer;
    Help1: TMenuItem;
    Panel_Report_Written: TPanel;
    Panel_Ref_Background: TPanel;
    Panel_Star_Background: TPanel;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Open1Click(Sender: TObject);
    procedure ComboBox_ScaleChange(Sender: TObject);
    procedure Button_PlusClick(Sender: TObject);
    procedure Button_MinusClick(Sender: TObject);
    procedure Button_VisualisationClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure ShowHeader1Click(Sender: TObject);
    procedure Close1Click(Sender: TObject);
    procedure Close2Click(Sender: TObject);
    procedure StringGrid_FITsClick(Sender: TObject);
    procedure LoadData1Click(Sender: TObject);
    procedure CloseAll1Click(Sender: TObject);
    procedure Settings1Click(Sender: TObject);
    procedure Button_AAVSOClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Help1Click(Sender: TObject);

  private
    IniFName        : String;
    DataFileName    : String;
    TargetFName     : String;
    Stringin        : String;
    Spos            : Integer;
    procedure Load_Data();
    function  Measure(): Boolean;
    function  GetString(): String;
    function  LocalAppDataPath : String;

  public
    Scale_Index : Integer;
    Targets     : Array[1..MAX_OBJECTS] of TTargetData;
    NTargets    : Integer;
    Radius      : Cardinal;
    Annulus     : Cardinal;
    Gap         : Cardinal;
    Readout_Noise : Real;
    Dark_Current  : Real;
    Gain          : Real;
    degC          : Real;
    Max_Error     : Real;
    ReportFileDir : String;
    AAVSO_ObsCode : String;
    Notes         : String;
    Snap_to_Centre: Boolean;
    Show_Annulus  : Boolean;

    procedure Hide_Zoom;
    procedure Show_Zoom;
    procedure Show_Results;
    procedure Trigger_Results;
    procedure Clear_Top_Panels;
    procedure Clear_Bottom_Panels;

    // Automation interfaces:
    //
    procedure SetFileName(FN: String);
    procedure SetGain (Value: Single);
    procedure SetdegC (Value: Single);
    procedure SetReadout_Noise (Value: Single);
    procedure SetDark_Current (Value: Single);
    procedure CloseImage();
    procedure SetMax_Error(Value: Single);
    procedure AAVSO_Report();

    function  GetMagnitude(): Double;
    function  GetLastError(): Integer;
    function  GetThreshold(): Double;
    function  GetGain(): Single;
    function  GetdegC(): Single;
    function  GetReadout_Noise(): Single;
    function  GetDark_Current(): Single;
    function  GetStandard_Deviation(): Single;
    function  GetLess_Than(): Integer;
    function  GetMax_Error(): Single;
  end;

var
  Main_Form: TMain_Form;

implementation

uses
  Math,StrUtils,Unit_Image, Unit_Settings, Unit_Help, Shlobj;

{$R *.dfm}

//////////////////////////////////////////////////////////////////
// Automation Interfaces:                                       //
//                                                              //
procedure TMain_Form.SetFileName(FN: String);                   //
begin
  TargetFName  := FN;
end;
  
procedure TMain_Form.SetGain (Value: Single);
begin
  Gain  := Value;
end;  

procedure TMain_Form.SetReadout_Noise (Value: Single);
begin
  Readout_Noise  := Value;
end;  

procedure TMain_Form.SetDark_Current (Value: Single);
begin
  Dark_Current  := Value;
end;  

procedure TMain_Form.SetdegC (Value: Single);
begin
  degC  := Value;
end;  

procedure TMain_Form.SetMax_Error (Value: Single);
begin
  Max_Error  := Value;
end;  

procedure TMain_Form.AAVSO_Report();
begin
  Button_AAVSOClick(Self);
end;

function TMain_Form.GetGain(): Single;
begin
  GetGain := Gain;
end;

function TMain_Form.GetReadout_Noise(): Single;
begin
  GetReadout_Noise := Readout_Noise;
end;

function TMain_Form.GetDark_Current(): Single;
begin
  GetDark_Current := Dark_Current;
end;

function TMain_Form.GetdegC(): Single;
begin
  GetdegC := degC;
end;

function TMain_Form.GetMax_Error(): Single;
begin
  GetMax_Error := Max_Error;
end;

function TMain_Form.GetMagnitude(): Double;
var
  Kid : TForm_Image;
begin
  if FileExists(TargetFName) then
  begin  
    Kid	:= TForm_Image.Create(Application);
    Kid.Open_File(TargetFName);
    if Measure() then
      GetMagnitude  := Kid.Magnitude
    else  
      GetMagnitude  := 0.0;
  end else
    GetMagnitude  := 0.0;
end;

// Errors:
//  0 Success
//  1 No active image
//  2 Not plate solved
//  3 Star Flux not measured
//  4 Reference Flux not measured
//

function TMain_Form.GetLastError(): Integer;
var
  Kid : TForm_Image;
begin
  Kid := TForm_Image(ActiveMDIChild);
  if not(Kid = nil) then
    GetLastError  := Kid.Last_Error
  else    
    GetLastError  := ERR_NO_ACTIVE_IMAGE;
end;

function TMain_Form.GetThreshold(): Double;
var
  Kid : TForm_Image;
begin
  Kid := TForm_Image(ActiveMDIChild);
  if not(Kid = nil) then
    GetThreshold  := Kid.Threshold
  else
    GetThreshold  := 0.0;
end;

function  TMain_Form.GetStandard_Deviation(): Single;
var
  Kid : TForm_Image;
begin
  Kid := TForm_Image(ActiveMDIChild);
  if not(Kid = nil) then
    GetStandard_Deviation := Kid.Std_Error
  else
    GetStandard_Deviation := 0.0;
end;

function  TMain_Form.GetLess_Than(): Integer;
var
  Kid : TForm_Image;
begin
  Kid := TForm_Image(ActiveMDIChild);
  if not(Kid = nil) then
    GetLess_Than := Kid.Less_Than
  else
    GetLess_Than := 0;
end;  

procedure TMain_Form.CloseImage();
begin
  Close2Click(Self);                                            //
end;                                                            //  
//////////////////////////////////////////////////////////////////

function TMain_Form.LocalAppDataPath : String;
const
  SHGFP_TYPE_CURRENT = 0;
var
  path  : array [0..MAX_PATH] of char;
begin
  SHGetFolderPath(0,CSIDL_LOCAL_APPDATA,0,SHGFP_TYPE_CURRENT,@path[0]);
  Result  := path;
end;

procedure TMain_Form.FormCreate(Sender: TObject);
// On creation of the main form, this routine reads and initialises the
// operational parameters from the application .ini file.  If no .ini file
// exists, the parameters are set to default values embedded in this routine.
var
  IniF		: TextFile;
  TempInt	: Integer;
  FString : String;
begin
  Timer1.Enabled  := False;
  IniFName := LocalAppDataPath + INIFILENAME;
  if FileExists(IniFName) then
  begin
    AssignFile(IniF, IniFName);
    FileMode := 0; {readonly}
    Reset(IniF);
    Readln(IniF);				{Title}
    Readln(IniF, TempInt); Top    := TempInt;
    Readln(IniF, TempInt); Left   := TempInt;
    Readln(IniF, TempInt); Width  := TempInt;
    Readln(IniF, TempInt); Height := TempInt;
    Readln(IniF, TempInt); Scale_Index := TempInt;
    Readln(IniF, Radius);
    Readln(IniF, Annulus);
    Readln(IniF, Gap);
    Readln(IniF, DataFileName);
    Readln(IniF, Readout_Noise);
    Readln(IniF, Dark_Current);
    Readln(IniF, Gain);
    Readln(IniF, degC);
    Readln(IniF, Max_Error);
    Readln(IniF, ReportFileDir);
    Readln(IniF, AAVSO_ObsCode);
    Readln(IniF, Notes);
    Readln(IniF, FString);
    if LeftStr(FString,1) = 'T' then
      Snap_to_Centre  := True
    else
      Snap_to_Centre  := False;
    Readln(IniF, FString);
    if LeftStr(FString,1) = 'T' then
      Show_Annulus  := True
    else
      Show_Annulus  := False;
    CloseFile(IniF);
  end else
  begin
    Top			    := 100;
    Left		    := 100;
    Width       := 100;
    Height      := 100;
    Scale_Index := 7;
    Radius      := 12;
    Annulus     := 3;
    Gap         := 3;
    DataFileName:= 'c:\data.csv';
    Readout_Noise := 0.0;
    Dark_Current  := 0.0;
    Gain          := 0.0;
    degC          := 0.0;
    Max_Error     := 1.0;   // magnitudes
    ReportFileDir := 'c:\temp';
    AAVSO_ObsCode := '';
    Notes         := 'na';
    Snap_to_Centre:= False;
    Show_Annulus  := False;
  end;
  ComboBox_Scale.ItemIndex  := Scale_Index;
  Hide_Zoom();
  StringGrid_FITs.Visible	:= False;
  Panel_Targets.Caption := 'No objects Loaded';
  NTargets  := 0;
  Load_Data();
  TargetFName  := '';
  Button_AAVSO.Visible  := False;
end;

procedure TMain_Form.FormDestroy(Sender: TObject);
begin
  Timer1.Enabled  := False;
end;

procedure TMain_Form.FormClose(Sender: TObject; var Action: TCloseAction);
// this routine close the application, saving current settings to the
// application .ini file
var
  IniF       : TextFile;
begin
{$I-}
  IniFName := LocalAppDataPath + INIFILENAME;
  AssignFile(IniF, IniFName);
  Rewrite(IniF);
{$I+}
  if IOResult = 0 then
  begin
    Writeln(IniF, '[Photometry Express]');
    Writeln(IniF, Top);
    Writeln(IniF, Left);
    Writeln(IniF, Width);
    Writeln(IniF, Height);
    Writeln(IniF, Scale_Index);
    Writeln(IniF, Radius);
    Writeln(IniF, Annulus);
    Writeln(IniF, Gap);
    Writeln(IniF, DataFileName);
    Writeln(IniF, Readout_Noise);
    Writeln(IniF, Dark_Current);
    Writeln(IniF, Gain);
    Writeln(IniF, degC);
    Writeln(IniF, Max_Error);
    Writeln(IniF, ReportFileDir);
    Writeln(IniF, AAVSO_ObsCode);
    Writeln(IniF, Notes);
    if Snap_to_Centre then
      Writeln(IniF, 'True')
    else
      Writeln(IniF, 'False');
    if Show_Annulus then
      Writeln(IniF, 'True')
    else
      Writeln(IniF, 'False');
    CloseFile(IniF);
  end;
end;

procedure TMain_Form.Clear_Top_Panels();
begin
  Panel_Image_Filter.Caption  := '';
  Panel_Plate_Solved.Caption  := '';
  Panel_Chart.Caption         := '';
  Panel_Magnitude.Caption     := '';
  Panel_Reference.Caption     := '';
  Panel_STDEV.Caption         := '';
  Panel_Outburst.Caption      := '';
end;

procedure TMain_Form.Clear_Bottom_Panels();
begin
  Panel_BP.Caption              := '';
  Panel_WP.Caption              := '';
  Panel_Location.Caption        := '';
  Panel_Pixel.Caption           := '';
  Panel_Position.Caption        := '';
  Panel_Errors.Caption          := '';
  Panel_Report_Written.Caption  := '';
  Panel_Ref_Background.Caption  := '';
  Panel_Star_Background.Caption := '';
end;

procedure TMain_Form.Show_Results();
var
  Kid       : TForm_Image;
begin
  Kid := TForm_Image(ActiveMDIChild);
  if not(Kid = nil) then
  case Kid.Last_Error of
    ERR_SUCCESS: begin  
        Panel_Reference.Caption := format('ref %2.3f%s',[Kid.Reference_Mag,Kid.Ref_Filter_Letter]);
        if Kid.Less_Than = 0 then
        begin  
          Panel_Magnitude.Caption := format('mag %2.3f',[Kid.Magnitude]);
          Panel_STDEV.Caption     := format('stdev %2.3f',[Kid.Std_Error])
        end else  
        begin  
          Panel_Magnitude.Caption := format('mag <%2.3f',[Kid.Magnitude]);
          Panel_STDEV.Caption     := format('stdev %2.3f',[Max_Error]);
        end;  
        Panel_Plate_Solved.Caption := 'Plate Solved';
        if Kid.Outburst then begin
          Panel_Outburst.Font.Color := clRed;
          Panel_Outburst.Caption := 'Outburst'
        end else begin  
          Panel_Outburst.Font.Color := clGreen;
          Panel_Outburst.Caption := 'Quiescent';
        end;  
        Panel_Chart.Caption     := 'Chart: ' + Kid.ChartName;
        Panel_Image_Filter.Caption  := Kid.Image_Filter_Letter + ' filter';
        Button_AAVSO.Visible    := True;
      end;

    ERR_NO_ACTIVE_IMAGE: ;        // no active image  

    ERR_NOT_PLATE_SOLVED: begin
        Clear_Top_Panels;
        Panel_Errors.Font.Color := clRed;
        Panel_Errors.Caption    := 'Not Plate Solved';
        Button_AAVSO.Visible    := False;
      end;  

    ERR_STAR_FLUX_NOT_MEASURED: begin
        Clear_Top_Panels;
        Panel_Errors.Font.Color := clRed;
        Panel_Errors.Caption    := 'No star!';
        Button_AAVSO.Visible    := False;
      end;  

    ERR_REF_FLUX_NOT_MEASURED: begin
        Clear_Top_Panels;
        Panel_Errors.Caption := '';
        Panel_Reference.Caption := 'No reference!';
        Panel_Plate_Solved.Caption := 'Plate Solved';
        Panel_Chart.Caption     := 'Chart: ' + Kid.ChartName;
        Button_AAVSO.Visible    := False;
      end;  

    ERR_NO_REFERENCE_DATA: begin
        Clear_Top_Panels;
        Panel_Errors.Caption := '';
        Panel_Reference.Caption := 'No data!';
        Panel_Plate_Solved.Caption := 'Plate Solved';
        Button_AAVSO.Visible    := False;
      end;  

    ERR_NO_REPORT_DIR: begin
        Clear_Top_Panels;
        Panel_Errors.Font.Color := clRed;
        Panel_Errors.Caption    := 'No report dir!';
        Button_AAVSO.Visible    := False;
      end;  
  end  
  else begin  // no active image
    Clear_Top_Panels;
    Button_AAVSO.Visible     := False;
  end;  
end;    

function TMain_Form.Measure(): Boolean;
var
  Kid       : TForm_Image;
  j         : Integer;
  pos       : TSkyPosition;
  IRA,IDec  : Double;
  dRA,dDec  : Double;
  distance  : Double;
  mindist   : Double;
  index     : Integer;
  name      : String;
  Success   : Boolean;

begin
  Kid := TForm_Image(ActiveMDIChild);
  if not Kid.Plate_Solved then
  begin
    Kid.Last_Error  := ERR_NOT_PLATE_SOLVED;
    Exit;
  end;
  Success := True;
  if NTargets > 0 then
  begin  
    if Kid.WCS1.coorflip then
    begin
      IRA   := Kid.WCS1.crval[1]; // image centre
      IDec  := Kid.WCS1.crval[0]; // ditto
    end else
    begin  
      IRA   := Kid.WCS1.crval[0]; // image centre
      IDec  := Kid.WCS1.crval[1]; // ditto
    end;

    mindist := 99999;
    index   := 1;
    for j:=1 to NTargets do     // find nearest target
    begin
      dRA  := 15.0*Targets[j].RA  - IRA;
      dDec := Targets[j].Dec - IDec;
      distance  := sqrt(dRA*dRA + dDec*dDec);           
      if distance < mindist then
      begin
        mindist := distance;
        index   := j;
        name    := Targets[j].Name;
      end;  
    end;  

    pos.RA  := Targets[index].RA;
    pos.Dec := Targets[index].Dec;
    Kid.Mark_Object(pos,name);
    if not(Targets[index].RName = '') then
    begin
      Kid.ChartName := Targets[index].Chartname;
      pos.RA  := Targets[index].RRA;
      pos.Dec := Targets[index].RDec;
      Kid.Mark_Reference(pos,Targets[index].RName,Targets[index].Rmag,Targets[index].RFilter);
      if Kid.Measure_Magnitude() then
      begin  
        if Kid.Last_Error = ERR_SUCCESS then
        begin  
          Kid.Threshold := Targets[index].Threshold;
          if (Kid.Less_Than = 0) and (Kid.Magnitude < Targets[index].Threshold) then
          begin  
            Kid.Outburst  := True
          end else begin  
            Kid.Outburst  := False;
          end;  
        end;  
      end else begin    
        Kid.Outburst  := False;
        Success := False;
      end;  
      Show_Results();
    end else
    begin
      Success := False;
      Kid.Last_Error := ERR_NO_REFERENCE_DATA;
      Show_Results();
    end  
  end else
    Success := False;
  Measure := Success;  
end;

procedure TMain_Form.Open1Click(Sender: TObject);
// This event handler procedure is called when user selects File -> Open Image
// from the main menu.  The filke choser dialogue allows multiple files to be
// selected.  Files are opened in turn and checked to ensure that the image is
// plate solved.
var
  i   : Integer;
  Kid : TForm_Image;
begin
  OpenDialog1.DefaultExt:= 'fit';
  OpenDialog1.Filter		:= 'FITs files (*.fit)|*.FIT';
  OpenDialog1.Options   := [ofAllowMultiSelect,ofEnableSizing,ofFileMustExist];
  if OpenDialog1.Execute then
  begin
    with OpenDialog1.Files do
      for i:=0 to Count-1 do
      begin
        Kid	:= TForm_Image.Create(Application);
        Kid.Open_File(Strings[i]);
        if Kid.Plate_Solved then
          Measure()
        else begin
          Clear_Top_Panels;
          Panel_Plate_Solved.Caption := 'Not Plate Solved';
          Button_AAVSO.Visible    := False;
        end;
      end;
    Show_Zoom();
  end;
end;

procedure TMain_Form.ComboBox_ScaleChange(Sender: TObject);
begin
  TForm_Image(ActiveMDIChild).Rescale(ComboBox_Scale.ItemIndex);
  Scale_Index := ComboBox_Scale.ItemIndex;
end;

procedure TMain_Form.Button_PlusClick(Sender: TObject);
begin
  if Scale_Index < 14 then
  begin
    Scale_Index := Scale_Index + 1;
    ComboBox_Scale.ItemIndex := Scale_Index;
    TForm_Image(ActiveMDIChild).ReScale(Scale_Index);
  end  
end;

procedure TMain_Form.Button_MinusClick(Sender: TObject);
begin
  if Scale_Index > 0 then
  begin
    Scale_Index := Scale_Index - 1;
    ComboBox_Scale.ItemIndex := Scale_Index;
    TForm_Image(ActiveMDIChild).ReScale(Scale_Index);
  end  
end;

procedure TMain_Form.Button_VisualisationClick(Sender: TObject);
begin
  TForm_Image(ActiveMDIChild).Reset_Visualisation;
end;

procedure TMain_Form.FormActivate(Sender: TObject);
var
  i: Integer;
begin
  for i:= MDIChildCount-1 downto 0 do
    TForm_Image(MDIChildren[i]).FormPaint(Self);
end;

// Zoom controls only relevant when at least one image is visible:
//
procedure TMain_Form.Help1Click(Sender: TObject);
begin
  Form_Help.ShowModal;
end;

procedure TMain_Form.Hide_Zoom();
begin
  ComboBox_Scale.Visible	:= False;  
  ComboBox_Scale.Enabled	:= False;  
  Button_Plus.Visible		:= False;
  Button_Plus.Enabled		:= False;
  Button_Minus.Visible		:= False;
  Button_Minus.Enabled		:= False;
  Button_Visualisation.Visible	:= False;
  Button_Visualisation.Enabled	:= False;
  FITs1.Enabled			:= False;
  Clear_Top_Panels;
end;

procedure TMain_Form.Show_Zoom();
begin
  ComboBox_Scale.Visible	:= True;
  ComboBox_Scale.Enabled	:= True;  
  Button_Plus.Visible		:= True;
  Button_Plus.Enabled		:= True;
  Button_Minus.Visible		:= True;
  Button_Minus.Enabled		:= True;
  Button_Visualisation.Visible	:= True;
  Button_Visualisation.Enabled	:= True;
  FITs1.Enabled			:= True;
end;

procedure TMain_Form.ShowHeader1Click(Sender: TObject);
var
  i,j,k	: Integer;
  Kid	: TForm_Image;
begin
  Kid := TForm_Image(ActiveMDIChild);
  with StringGrid_FITs do
  begin
    RowCount := Kid.NKeys+1;
    j := (RowCount+1) * 22;
    k := Main_Form.ClientHeight - 50;
    if j > k then
      Height := k
    else
      Height := j;
    Cells[0,0] := 'KEY';
    Cells[1,0] := 'VALUE';
    Cells[2,0] := 'COMMENT';
    for i:=0 to RowCount-1 do
    begin
      Cells[0,i+1] := Kid.KeyNames[i];
      Cells[1,i+1] := Kid.Values[i];
      Cells[2,i+1] := Kid.CommentStrings[i];
    end;
  end;
  StringGrid_FITs.Visible := True;
end;

// Close the FITs header:
//
procedure TMain_Form.Close1Click(Sender: TObject);
begin
  StringGrid_FITs.Visible := False;
end;

// Close an image:
//
procedure TMain_Form.Close2Click(Sender: TObject);
begin
  if ActiveMDIChild <> nil then
  begin
    ActiveMDIChild.Close;
    if MDIChildCount < 2 then
      Hide_Zoom();
  end;  
  Application.ProcessMessages;
  Show_Results();
end;

procedure TMain_Form.StringGrid_FITsClick(Sender: TObject);
begin
  StringGrid_FITs.Visible := False
end;

function TMain_Form.GetString(): String;
var
  ch    : Char;
  Done  : Boolean;
  Res   : String;
begin
  Res := '';
  Done      := False;
  Repeat
    if Spos = Length(Stringin)+1 then
      Done  := True
    else begin  
      ch    := Stringin[Spos];
      inc(Spos,1);
      if ch = ',' then
        Done := True
      else  
        Res := Res + ch;
    end;   
  Until Done;
  GetString := Res;
end;

procedure TMain_Form.Load_Data();
// this routine reads the objects data file specified by DataFileName and
// populates the target list array (Targets).  RA and DEC in Ddegrees
// minutes and seconds are converted to decimal.  The maximum number of
// targets is defined by constant MAX_OBJECTS.
//
// The objects csv data file format is as follows:
//
//     Object name
//     Object RA(HH)
//     Object RA(MM)
//     Object RA(SS)
//     Object Dec(Deg)
//     Object Dec(MM)
//     Object Dec(SS)
//     Reference star name
//     Reference star RA(HH)
//     Reference star RA(MM)
//     Reference star RA (SS)
//     Reference star Dec(HH)
//     Reference star Dec(MM)
//     Reference star Dec(SS)
//     Filter
//     Magnitude
//     Threshold
//     ChartName

var
  i         : Integer;
  F         : TextFile;
  HH,MM,SS  : String;
  NN        : String;
begin
  if not FileExists(DataFileName) then
    ShowMessage(DataFileName + ' not found')
  else
  begin  
    AssignFile(F, DataFileName);
    FileMode  := 0; {readonly}
    Reset(F);
    Readln(F); Readln(F); Readln(F);      // skip headers
    i         := 1;
    NTargets  := 0;
    While (not EOF(F)) and (i < MAX_OBJECTS+1) do
    begin
      Readln(F, Stringin);
      if Length(Stringin) > 0 then        // allow empty lines
      begin
        Spos  := 1;
        Targets[i].Name   := GetString;
        if Length(Targets[i].Name) > 0 then
          inc(NTargets,1);
        HH  := GetString;
        MM  := GetString;
        SS  := GetString;
        Targets[i].RA := StrtoInt(HH) + StrtoInt(MM)/60.0 + StrtoFloat(SS)/3600.0;
        HH  := GetString;
        MM  := GetString;
        SS  := GetString;
        if LeftStr(HH,1) = '-' then
          Targets[i].Dec := StrtoInt(HH) - StrtoInt(MM)/60.0 - StrtoFloat(SS)/3600.0
        else  
          Targets[i].Dec := StrtoInt(HH) + StrtoInt(MM)/60.0 + StrtoFloat(SS)/3600.0;
    
        // Now get data for the reference star, if any:
        //
        NN  := GetString;
        Targets[i].RName  := NN;
        if Length(NN) > 0 then
        begin  
          HH  := GetString;
          MM  := GetString;
          SS  := GetString;
          Targets[i].RRA := StrtoInt(HH) + StrtoInt(MM)/60.0 + StrtoFloat(SS)/3600.0;
          HH  := GetString;
          MM  := GetString;
          SS  := GetString;
          if LeftStr(HH,1) = '-' then
            Targets[i].RDec := StrtoInt(HH) - StrtoInt(MM)/60.0 - StrtoFloat(SS)/3600.0
          else  
            Targets[i].RDec := StrtoInt(HH) + StrtoInt(MM)/60.0 + StrtoFloat(SS)/3600.0;
          NN  := GetString;
          Targets[i].RFilter  := NN[1];
          Targets[i].Rmag :=StrtoFloat(GetString);  
          NN := GetString;
          if Length(NN) > 0 then
            try
              Targets[i].Threshold := StrtoFLoat(NN);
            except
              on EConvertError do
                Targets[i].Threshold := 0.0;
            end;
          NN  := GetString;
          Targets[i].ChartName  := NN;
        end else
        begin
          Targets[i].RRA        := 0.0;
          Targets[i].RDec       := 0.0;
          Targets[i].Rmag       := 0.0;
          Targets[i].ChartName  := '';
        end;  
        inc(i,1);
      end;  
    end;
    if not EOF(F) then
      Panel_Targets.Caption := format('Too many (%d) Targets',[NTargets])
    else  
      Panel_Targets.Caption := format('%d Targets Loaded',[NTargets]);
    CloseFile(F);
  end;  
end;

procedure TMain_Form.LoadData1Click(Sender: TObject);
// This routine reloads the Object data by calling the Load_Data routine
begin
  OpenDialog1.Title       := 'Load Targets';
  OpenDialog1.Filter		  := 'csv files (*.csv)|*.csv';
  OpenDialog1.DefaultExt	:= 'csv';
  OpenDialog1.InitialDir	:= '.';
  if OpenDialog1.Execute then
  begin
    DataFileName  := OpenDialog1.FileName;
    Load_Data()
  end else
    Panel_Targets.Caption := 'No objects Loaded';
end;

procedure TMain_Form.CloseAll1Click(Sender: TObject);
var
  NKids : Integer;
  i     : Integer;
  Kid	  : TForm_Image;
begin
  NKids := MDIChildCount;
  for i := NKids-1 downto 0 do
  begin
    Kid := TForm_Image(MDIChildren[i]);
    if not(Kid = nil) then
      Kid.Close;
  end;
  Hide_Zoom();
end;

procedure TMain_Form.Settings1Click(Sender: TObject);
begin
  Form_Settings.Show;
end;

// Write a report in AAVSO format:
//
procedure TMain_Form.Button_AAVSOClick(Sender: TObject);
var
  DTString	: String;
  ReportF   : TextFile;
  Kid	      : TForm_Image;
  i         : Integer;
begin
  Kid := TForm_Image(ActiveMDIChild);
  if not(Kid = nil) then
  begin
    DTString  := Kid.Date_Obs;
    DTString  := LeftStr(DTString,Length(DTString)-2);
    DTString  := RightStr(DTString,Length(DTString)-1);
    for i:= 1 to Length(DTString) do
      if DTString[i] = ':' then
        DTString[i] := '_';

{$I-}
    AssignFile(ReportF, ReportFileDir + '/' + Kid.ObjectName + '_' + DTString
                          + '.txt');
    Rewrite(ReportF);
{$I+}
    if IOResult = 0 then
    begin
      Writeln(ReportF, '#TYPE=EXTENDED');
      Writeln(ReportF, '#OBSCODE=' + AAVSO_ObsCode);
      Writeln(ReportF, '#SOFTWARE=Photometry Express V1.0');
      Writeln(ReportF, '#DELIM=,');
      Writeln(ReportF, '#DATE=JD');
      Writeln(ReportF, '#OBSTYPE=CCD');
      Writeln(ReportF, '#NAME,DATE,MAG,MERR,FILT,TRANS,MTYPE,CNAME,CMAG,KNAME,KMAG,AMASS,GROUP,CHART,NOTES');
      Write(ReportF, Kid.ObjectName +',');
      Write(ReportF,format('%7.5f,',[Kid.JD]));
      if Kid.Less_Than = 0 then
      begin  
        Write(ReportF,format('%2.3f,',[Kid.Magnitude]));
        Write(ReportF,format('%2.3f,',[Kid.Std_Error]));
      end else begin  
        Write(ReportF,format('<%2.3f,',[Kid.Magnitude]));
        Write(ReportF,format('%2.3f,',[Max_Error]));
      end;  
      if Kid.Image_Filter_Letter = Kid.Ref_Filter_Letter then
        Write(ReportF, Kid.Image_Filter_Letter + ',')
      else begin
        if (Kid.Image_Filter_Letter = 'L')  and (Kid.Ref_Filter_Letter = 'V') then
          Write(ReportF, 'CV,')
        else  
          ShowMessage(format('Image Filter = %s, Ref Filter = %s',[Kid.Image_Filter_Letter,Kid.Ref_Filter_Letter]));
      end;    
      Write(ReportF, 'NO,STD,');
      Write(ReportF, Kid.ReferenceName + ',');
      Write(ReportF,format('%2.3f,',[Kid.CMag]));
      Write(ReportF, 'na,na,');
      Write(ReportF,format('%2.3f,',[Kid.AirMass]));
      Write(ReportF, 'na,');
      Write(ReportF, Kid.ChartName + ',');
      Writeln(ReportF, Notes + format(' %4.1f seconds',[Kid.Exposure]));
      CloseFile(ReportF);
      Panel_Report_Written.Caption := 'Report written';
      Kid.AAVSO_Written   := True;
    end else
    begin
      Kid.Last_Error := ERR_NO_REPORT_DIR;
      Panel_Errors.Font.Color := clRed;
      Panel_Errors.Caption := 'Report dir not found!';
      Kid.AAVSO_Written   := False;
    end;  
  end;
end;

procedure TMain_Form.Timer1Timer(Sender: TObject);
begin
  Timer1.Enabled  := False;
  Show_Results();
end;

procedure TMain_Form.Trigger_Results;
begin
  Timer1.Interval := 500;
  Timer1.Enabled  := True;
end;

end.
