//----------------------------------------------------------------------------
//
// Module: unit_main.pas (Part of Photometry Express)
// Author: J C Moore, Fleet Electronics Consultancy
//
// Copyright (C) 2013  J C Moore
//
// NB before Windows 7 this program will register itself automatically.
// For Win 7 onwards proceed as follows:
// Start a command prompt as administrator, change to the Photometry Express directory and type:
//    'Photometry_Express.exe /regserver'
//    (it's silent whether it succedes or not)
//
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation, either version 3 of
// the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// see http://www.gnu.org/licenses/
//
// Update History:
//    01 May 2012: last update by J C Moore
//
//
//----------------------------------------------------------------------------
{ Photometry Express
  Unit to create a COM server
  by J C Moore
  Fleet Electronics Consultancy
  Last update 01 May 2012
  NB before Windows 7 this program will register itself automatically.
  Fomr Win 7 inwards proceed as follows:
  Start a command prompt as administrator, change to the Photometry Express directory and type:
    'Photometry_Express.exe /regserver'
    (it's silent whether it succedes or not)
  }

unit Unit_Server;

{$WARN SYMBOL_PLATFORM OFF}

interface

uses
  ComObj, ActiveX, Photometry_Express_TLB, StdVcl;

type
  TPhoto_Express_Server = class(TAutoObject, IPhoto_Express_Server)
  protected
    procedure Set_FileName(const Param1: WideString); safecall;
    function  Get_Magnitude: Double; safecall;
    function Get_LastError: SYSINT; safecall;
    procedure CloseFile; safecall;
    function Get_Threshold: Double; safecall;
    procedure Set_Gain(Value: Single); safecall;
    function Get_Gain: Single; safecall;
    function Get_degC: Single; safecall;
    procedure Set_degC(Value: Single); safecall;
    function Get_Dark_Current: Single; safecall;
    function Get_Readout_Noise: Single; safecall;
    procedure Set_Dark_Current(Value: Single); safecall;
    procedure Set_Readout_Noise(Value: Single); safecall;
    function Get_Less_Than: SYSINT; safecall;
    function Get_Standard_Deviation: Single; safecall;
    function Get_Max_Error: Single; safecall;
    procedure Set_Max_Error(Value: Single); safecall;
    procedure AAVSO_Report; safecall;

  end;

implementation

uses ComServ,Unit_Main;

procedure TPhoto_Express_Server.Set_FileName(const Param1: WideString);
begin
  Main_Form.SetFileName(Param1);
end;

function TPhoto_Express_Server.Get_Magnitude: Double;
begin
  Get_Magnitude := Main_Form.GetMagnitude();
end;

function TPhoto_Express_Server.Get_Readout_Noise: Single;
begin
  Get_Readout_Noise := Main_Form.GetReadout_Noise();
end;

function TPhoto_Express_Server.Get_Dark_Current: Single;
begin
  Get_Dark_Current := Main_Form.GetDark_Current();
end;

function TPhoto_Express_Server.Get_LastError: SYSINT;
begin
  Get_LastError  := Main_Form.GetLastError;
end;

procedure TPhoto_Express_Server.CloseFile;
begin
  Main_Form.CloseImage();
end;

function TPhoto_Express_Server.Get_Threshold: Double;
begin
  Get_Threshold := Main_Form.GetThreshold;
end;

procedure TPhoto_Express_Server.Set_Gain(Value: Single);
begin
  Main_Form.SetGain(Value);
end;

procedure TPhoto_Express_Server.Set_Readout_Noise(Value: Single);
begin
  Main_Form.SetReadout_Noise(Value);
end;

procedure TPhoto_Express_Server.Set_Dark_Current(Value: Single);
begin
  Main_Form.SetDark_Current(Value);
end;

function TPhoto_Express_Server.Get_Gain: Single;
begin
  Get_Gain  := Main_Form.GetGain;
end;

function TPhoto_Express_Server.Get_degC: Single;
begin
  Get_degC  := Main_Form.GetdegC;
end;

procedure TPhoto_Express_Server.Set_degC(Value: Single);
begin
  Main_Form.SetdegC(Value);
end;

function TPhoto_Express_Server.Get_Less_Than: SYSINT;
begin
  Get_Less_Than := Main_Form.GetLess_Than;
end;

function TPhoto_Express_Server.Get_Standard_Deviation: Single;
begin
  Get_Standard_Deviation  := Main_Form.GetStandard_Deviation;
end;

function TPhoto_Express_Server.Get_Max_Error: Single;
begin
  Get_Max_Error  := Main_Form.GetMax_Error;
end;

procedure TPhoto_Express_Server.Set_Max_Error(Value: Single);
begin
  Main_Form.SetMax_Error(Value);
end;

procedure TPhoto_Express_Server.AAVSO_Report;
begin
  Main_Form.AAVSO_Report();
end;

initialization
  TAutoObjectFactory.Create(ComServer, TPhoto_Express_Server, Class_Photo_Express_Server,
    ciSingleInstance, tmApartment);
end.
