//----------------------------------------------------------------------------
//
// Module: unit_main.pas (Part of Photometry Express)
// Author: J C Moore, Fleet Electronics Consultancy
//
// Copyright (C) 2013  J C Moore
//
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation, either version 3 of
// the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// see http://www.gnu.org/licenses/
//
// Update History:
//    05 Mar 2013: last update by J C Moore
//
//
//----------------------------------------------------------------------------

unit Unit_Settings;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Spin;

type
  TForm_Settings = class(TForm)
    Label1              : TLabel;
    Label2              : TLabel;
    Label3              : TLabel;
    Label4              : TLabel;
    Label5              : TLabel;
    Label6              : TLabel;
    Label7              : TLabel;
    Label8              : TLabel;
    Label9              : TLabel;
    Label10             : TLabel;
    Label11             : TLabel;
    Label13             : TLabel;
    Label14             : TLabel;
    Label15             : TLabel;
    Label16             : TLabel;
    Edit_Readout_Noise  : TEdit;
    Edit_Dark_Current   : TEdit;
    Edit_Gain           : TEdit;
    Edit_degC           : TEdit;
    Edit_Max_Error      : TEdit;
    Edit_Report_Directory: TEdit;
    Edit_AAVSO_ObsCode  : TEdit;
    Edit_Notes          : TEdit;
    SpinEdit_Radius     : TSpinEdit;
    SpinEdit_Annulus    : TSpinEdit;
    SpinEdit_Gap        : TSpinEdit;
    CheckBox_Snap_to_Centre: TCheckBox;
    CheckBox_Show_Annulus: TCheckBox;

    procedure Edit_Readout_NoiseChange(Sender: TObject);
    procedure Edit_Dark_CurrentChange(Sender: TObject);
    procedure Edit_GainChange(Sender: TObject);
    procedure Edit_degCChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Edit_Max_ErrorChange(Sender: TObject);
    procedure Edit_Report_DirectoryChange(Sender: TObject);
    procedure Edit_AAVSO_ObsCodeChange(Sender: TObject);
    procedure Edit_NotesChange(Sender: TObject);
    procedure SpinEdit_RadiusChange(Sender: TObject);
    procedure SpinEdit_AnnulusChange(Sender: TObject);
    procedure SpinEdit_GapChange(Sender: TObject);
    procedure CheckBox_Snap_to_CentreClick(Sender: TObject);
    procedure CheckBox_Show_AnnulusClick(Sender: TObject);

  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form_Settings: TForm_Settings;

implementation

uses Unit_Main,Unit_Image;

{$R *.dfm}

procedure TForm_Settings.Edit_Readout_NoiseChange(Sender: TObject);
begin
  try
    Main_Form.Readout_Noise := StrtoFloat(Edit_Readout_Noise.Text);
  except
    on EConvertError do
      Main_Form.Readout_Noise := 0;
  end;    
end;

procedure TForm_Settings.Edit_Dark_CurrentChange(Sender: TObject);
begin
  try
    Main_Form.Dark_Current := StrtoFloat(Edit_Dark_Current.Text);
  except
    on EConvertError do
      Main_Form.Dark_Current  := 0;
  end;    
end;

procedure TForm_Settings.Edit_GainChange(Sender: TObject);
begin
  try
    Main_Form.Gain := StrtoFloat(Edit_Gain.Text);
  except
    on EConvertError do
      Main_Form.Gain  := 0;
  end;    
end;

procedure TForm_Settings.Edit_degCChange(Sender: TObject);
begin
  try
    Main_Form.degC := StrtoFloat(Edit_degC.Text);
  except
    on EConvertError do
      Main_Form.degC  := 0;
  end;    
end;

procedure TForm_Settings.FormShow(Sender: TObject);
begin
  Edit_Readout_Noise.Text  := format('%3.2f',[Main_Form.Readout_Noise]);
  Edit_Dark_Current.Text  := format('%3.2f',[Main_Form.Dark_Current]);
  Edit_Gain.Text  := format('%3.2f',[Main_Form.Gain]);
  Edit_degC.Text  := format('%3.2f',[Main_Form.degC]);
  Edit_Max_Error.Text  := format('%2.2f',[Main_Form.Max_Error]);
  Edit_Report_Directory.Text  := Main_Form.ReportFileDir;
  Edit_AAVSO_ObsCode.Text   := Main_Form.AAVSO_ObsCode;
  Edit_Notes.Text           := Main_Form.Notes;
  SpinEdit_Radius.Value     := Main_Form.Radius;
  SpinEdit_Annulus.Value    := Main_Form.Annulus;
  SpinEdit_Gap.Value        := Main_Form.Gap;
  CheckBox_Snap_to_Centre.Checked := Main_Form.Snap_to_Centre;
  CheckBox_Show_Annulus.Checked   := Main_Form.Show_Annulus;
end;

procedure TForm_Settings.FormCreate(Sender: TObject);
begin
  Top := Main_Form.Top + 72;
  Left:= Main_Form.Left + 42;
end;

procedure TForm_Settings.Edit_Max_ErrorChange(Sender: TObject);
begin
  try
    Main_Form.Max_Error := StrtoFloat(Edit_Max_Error.Text);
  except
    on EConvertError do
      Main_Form.Max_Error  := 0;
  end;    
end;

procedure TForm_Settings.Edit_Report_DirectoryChange(Sender: TObject);
begin
  Main_Form.ReportFileDir  := Edit_Report_Directory.Text;
end;

procedure TForm_Settings.CheckBox_Show_AnnulusClick(Sender: TObject);
begin
  Main_Form.Show_Annulus := CheckBox_Show_Annulus.Checked;
end;

procedure TForm_Settings.CheckBox_Snap_to_CentreClick(Sender: TObject);
begin
  Main_Form.Snap_to_Centre := CheckBox_Snap_to_Centre.Checked;
end;

procedure TForm_Settings.Edit_AAVSO_ObsCodeChange(Sender: TObject);
begin
  Main_Form.AAVSO_ObsCode := Edit_AAVSO_ObsCode.Text;
end;

procedure TForm_Settings.Edit_NotesChange(Sender: TObject);
begin
  Main_Form.Notes := Edit_Notes.Text;
end;

procedure TForm_Settings.SpinEdit_RadiusChange(Sender: TObject);
var
  NKids : Integer;
  i     : Integer;
  Kid	  : TForm_Image;
begin
  Main_Form.Radius:= SpinEdit_Radius.Value;
  NKids := MDIChildCount;
  for i := NKids-1 downto 0 do
  begin  
    Kid := TForm_Image(MDIChildren[i]);
    if not(Kid = nil) then
      Kid.Fill_Bitmap();      // redraw circles with new radius
  end;    
end;

procedure TForm_Settings.SpinEdit_AnnulusChange(Sender: TObject);
begin
  Main_Form.Annulus:= SpinEdit_Annulus.Value;
end;

procedure TForm_Settings.SpinEdit_GapChange(Sender: TObject);
begin
  Main_Form.Gap:= SpinEdit_Gap.Value;
end;

end.
